package CDApp;

use strict;
use DBI;
use Data::Dumper;
use Glib qw(TRUE FALSE);


use base 'Class::Accessor::Fast';

__PACKAGE__->mk_accessors(qw(gladexml db discs genre disclist_model));


use constant COL_GENRE  => 0;

use constant COL_DISCID => 0;
use constant COL_ARTIST => 1;
use constant COL_ALBUM  => 2;

my $db_dsn  = 'DBI:CSV:f_dir=/home/grant/projects/talks/gtk-perl/data';
my $db_attr = { RaiseError => 1, };


sub new        { return bless({}, shift)->init; }

sub db_connect { shift->db(DBI->connect($db_dsn, undef, undef, $db_attr)); }

sub DESTROY    { shift->db->disconnect; }


sub init {
  my $self = shift;

  $self->db_connect;
  $self->load_data;

  return $self;
}


sub autoconnect {
  my($self, $method, $widget, $signal) = @_;

  $widget->signal_connect($signal => sub { $self->$method(@_) });
}


sub init_gui {
  my $self = shift;

  # Replace dummy option in Genre ComboBox
  my $combo = $self->gladexml->get_widget('genre') or die "No 'genre' combobox";
  my $gmodel = $combo->get_model or die "combobox has no storage";
  
  $gmodel->clear;
  foreach my $genre (@{$self->genre}) {
    my $iter = $gmodel->append;
    $gmodel->set($iter, COL_GENRE, $genre);
  }

  # Set up model and column views for DiscList listbox
  my $dmodel = Gtk2::ListStore->new(
    'Glib::String',      # COL_DISCID
    'Glib::String',      # COL_ARTIST
    'Glib::String',      # COL_ALBUM
  );
  $self->disclist_model($dmodel);

  my $disclist = $self->gladexml->get_widget('disclist')
    or die "No 'disclist' listbox";

  $disclist->set_model($dmodel);

  my($renderer, $column);
  $renderer = Gtk2::CellRendererText->new;
  $column   = Gtk2::TreeViewColumn->new_with_attributes (
    "Artist",
    $renderer,
    text => COL_ARTIST,
  );
  $column->set_resizable(TRUE);
  $disclist->append_column($column);

  $renderer = Gtk2::CellRendererText->new;
  $column   = Gtk2::TreeViewColumn->new_with_attributes (
    "Album",
    $renderer,
    text => COL_ALBUM,
  );
  $column->set_resizable(TRUE);
  $disclist->append_column($column);

  $combo->set_active(0);
}


sub load_data {
  my $self = shift;

  my $data = $self->db->selectall_hashref(
    'SELECT discid, artist, title, year, genre FROM cdlist', 'discid'
  );
  $self->discs($data);

  my %genre = map { $_->{genre} => 1 } values %$data;
  $self->genre( [ sort keys %genre ] );
}


sub on_genre_changed {
  my($self, $combo) = @_;

  my $model     = $combo->get_model or die "'genre' combo has no model";
  my $iter      = $combo->get_active_iter;
  my $selection = $model->get($iter, COL_GENRE);

  my $dmodel    = $self->disclist_model;
  $dmodel->clear;

  my @discs = grep { $_->{genre} eq $selection } values %{ $self->discs };

  foreach my $disc (@discs) {
    my $iter = $dmodel->append;
    $dmodel->set($iter, COL_DISCID, $disc->{discid});
    $dmodel->set($iter, COL_ARTIST, $disc->{artist});
    $dmodel->set($iter, COL_ALBUM,  $disc->{title} );
  }
}


sub on_disclist_cursor_changed {
  my($self, $disclist) = @_;

  my $model     = $disclist->get_model or die "'genre' combo has no model";
  my($path)     = $disclist->get_cursor;
  my $iter      = $model->get_iter($path);
  my $selection = $model->get($iter, COL_DISCID);

  my $album = $self->discs->{$selection} or return;

  foreach my $key (qw(artist title year discid)) {
    if(my $entry = $self->gladexml->get_widget($key)) {
      $entry->set_text($album->{$key} || '');
    }
  }
}


sub on_cdapp_delete_event {
  Gtk2->main_quit;
}

1;

